/**
 * Assignment #6: Page Replacement Algorithms
 * Sample Solution
 *
 * Instructor: Ron Mak
 * CMPE 142 Operating Systems
 * Computer Engineering Department
 * San Jose State University
 */
#ifndef REPLACEMENTALGORITHM_H_
#define REPLACEMENTALGORITHM_H_

#include <string>
#include <vector>

using namespace std;

class ReplacementAlgorithm
{
public:
    ReplacementAlgorithm(string name, vector<int>& ref_string,
                         vector<int>& frames, const int frame_count)
        : name(name),ref_string(ref_string),
          frames(frames), frame_count(frame_count),
          hit_count(0), swap_count(0)
    {}

    virtual ~ReplacementAlgorithm() {}

    string get_name() const { return name; }

    void run(const int run_index);

    virtual int    get_total_hit_count()  const = 0;
    virtual double get_total_swap_count() const = 0;

protected:
    string name;
    vector<int>& ref_string;
    vector<int>& frames;
    int frame_count;

    int which_frame(const int ref);

private:
    int    hit_count;
    double swap_count;

    void print_stats();

    virtual void init() = 0;
    virtual void load_page(const int ref) = 0;
    virtual int choose_victim_frame(const int ref_index) = 0;
    virtual void record_hit() = 0;
    virtual void record_swap(const double amount) = 0;
};

#endif /* REPLACEMENTALGORITHM_H_ */
